@@ -3,11 +3,13 @@ package jwt
 import (
 	"encoding/json"
 	"fmt"
-	"log"
 	"math/rand"
 	"net/http"
 	"strings"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
+
 	"github.com/openshift/telemeter/pkg/authorize"
 )
 
@@ -17,6 +19,7 @@ type authorizeClusterHandler struct {
 	expireInSeconds int64
 	signer          *Signer
 	clusterAuth     authorize.ClusterAuthorizer
+	logger          log.Logger
 }
 
 // NewAuthorizerHandler creates an authorizer HTTP endpoint that will authorize the cluster
@@ -26,13 +29,14 @@ type authorizeClusterHandler struct {
 // in a generated signed JWT which is returned to the client, along with any labels.
 //
 // A single partition key parameter must be passed to uniquely identify the caller's data.
-func NewAuthorizeClusterHandler(partitionKey string, expireInSeconds int64, signer *Signer, labels map[string]string, ca authorize.ClusterAuthorizer) *authorizeClusterHandler {
+func NewAuthorizeClusterHandler(logger log.Logger, partitionKey string, expireInSeconds int64, signer *Signer, labels map[string]string, ca authorize.ClusterAuthorizer) *authorizeClusterHandler {
 	return &authorizeClusterHandler{
 		partitionKey:    partitionKey,
 		expireInSeconds: expireInSeconds,
 		signer:          signer,
 		labels:          labels,
 		clusterAuth:     ca,
+		logger:          log.With(logger, "component", "authorize/jwt"),
 	}
 }
 
@@ -69,11 +73,10 @@ func (a *authorizeClusterHandler) ServeHTTP(w http.ResponseWriter, req *http.Req
 	clientToken := auth[1]
 
 	subject, err := a.clusterAuth.AuthorizeCluster(clientToken, cluster)
-
 	if err != nil {
 		if scerr, ok := err.(authorize.ErrorWithCode); ok {
 			if scerr.HTTPStatusCode() >= http.StatusInternalServerError {
-				log.Printf("error: unable to authorize request: %v", scerr)
+				level.Error(a.logger).Log("msg", "unable to authorize request", "err", scerr)
 			}
 			if scerr.HTTPStatusCode() == http.StatusTooManyRequests {
 				w.Header().Set("Retry-After", "300")
@@ -84,7 +87,7 @@ func (a *authorizeClusterHandler) ServeHTTP(w http.ResponseWriter, req *http.Req
 
 		// always hide errors from the upstream service from the client
 		uid := rand.Int63()
-		log.Printf("error: unable to authorize request %d: %v", uid, err)
+		level.Error(a.logger).Log("msg", "unable to authorize request", "uid", uid, "err", err)
 		http.Error(w, fmt.Sprintf("Internal server error, requestid=%d", uid), http.StatusInternalServerError)
 		return
 	}
@@ -99,7 +102,7 @@ func (a *authorizeClusterHandler) ServeHTTP(w http.ResponseWriter, req *http.Req
 	// create a token that asserts the client and the labels
 	authToken, err := a.signer.GenerateToken(Claims(subject, labels, a.expireInSeconds, []string{"federate"}))
 	if err != nil {
-		log.Printf("error: unable to generate token: %v", err)
+		level.Error(a.logger).Log("msg", "unable to generate token", "err", err)
 		http.Error(w, err.Error(), http.StatusInternalServerError)
 		return
 	}
@@ -113,12 +116,12 @@ func (a *authorizeClusterHandler) ServeHTTP(w http.ResponseWriter, req *http.Req
 	})
 
 	if err != nil {
-		log.Printf("error: unable to marshal token: %v", err)
+		level.Error(a.logger).Log("msg", "unable to marshal token", "err", err)
 		http.Error(w, err.Error(), http.StatusInternalServerError)
 		return
 	}
 
 	if _, err := w.Write(data); err != nil {
-		log.Printf("writing auth token failed: %v", err)
+		level.Error(a.logger).Log("msg", "writing auth token failed", "err", err)
 	}
 }
